@@ -1,19 +1,25 @@
+import hashlib
 import importlib
 import json
+import logging
 import os
 import re
 import shutil
+import sys
 import time
+import warnings
 from types import ModuleType
-from typing import List, Dict, Any, Tuple
+from typing import List, Dict, Any, Tuple, Union
 from urllib.parse import unquote
-import logging
 
 import requests
 
 from aicrowd.notebook import exceptions
-from aicrowd.utils.jupyter import is_google_colab_env, mount_google_drive
-
+from aicrowd.utils.jupyter import (
+    is_google_colab_env,
+    mount_google_drive,
+    is_ipykernel_present,
+)
 
 log = logging.getLogger()
 
@@ -116,11 +122,93 @@ def get_latest_jupyter_session() -> Dict[str, Any]:
 def get_notebook_path():
     if is_google_colab_env():
         log.debug("Detected google colab environment")
-        return get_colab_notebook_path()
+        return get_colab_notebook_via_gdrive()
     else:
         return get_jupyter_notebook_path()
 
 
+def get_notebook(notebook_path: str = None) -> Tuple[Union[Dict[str, Any], str], str]:
+    """Get notebook JSON or notebook file path and, notebook name.
+
+    If notebook path is provided then returns the notebook path along with the notebook name else based on the kernel
+    environment, returns notebook and notebook name. If the environment is Google colaboratory and if the code is
+    running inside kernel, fetch notebook from frontend else mount google drive and get notebook path. If the
+    environment is Jupyter notebook or Jupyterlab, get notebook path from disk. If the code is running inside kernel,
+    save the notebook else show warning message.
+
+    Args:
+        notebook_path: Path of the notebook
+
+    Returns:
+        notebook: A JSON dictionary containing notebook contents or Path to the notebook file
+        notebook_name: Name of the notebook
+    """
+    if notebook_path:
+        return notebook_path, os.path.basename(notebook_path)
+    if is_google_colab_env():
+        log.debug("Detected google colab environment")
+        return get_colab_notebook()
+    return get_saved_jupyter_notebook()
+
+
+def get_colab_notebook():
+    notebook, notebook_name = None, None
+    if is_ipykernel_present():
+        notebook, notebook_name = get_colab_notebook_via_kernel()
+    else:
+        show_warning_to_use_magic_command(reason="to skip mounting the drive")
+    if notebook is None:
+        notebook = get_colab_notebook_via_gdrive()
+        notebook_name = os.path.basename(notebook)
+    return notebook, notebook_name
+
+
+def get_saved_jupyter_notebook():
+    notebook_path = get_jupyter_notebook_path()
+    notebook_name = os.path.basename(notebook_path)
+    if is_ipykernel_present():
+        save_jupyter_notebook(notebook_path)
+    else:
+        show_warning_to_use_magic_command(reason="to save the notebook automatically")
+    return notebook_path, notebook_name
+
+
+def get_colab_notebook_via_kernel(
+    timeout_sec: int = 180,
+) -> Union[Tuple[Dict[str, Any], str], Tuple[None, None]]:
+    """
+    Get colab notebook from colab frontend via messaging
+    """
+    _message = import_module("google.colab._message")
+    notebook = _message.blocking_request("get_ipynb", timeout_sec=timeout_sec)
+    if notebook is None:
+        return None, None
+    return notebook["ipynb"], notebook["ipynb"]["metadata"]["colab"]["name"]
+
+
+def save_jupyter_notebook(file_path: str, timeout_sec: int = 5):
+    """Save jupyter notebook
+
+    Use kernel comms messaging to save the notebook and wait until the notebook is saved or timeout occurs.
+
+    Args:
+        file_path: Notebook file path to be saved
+        timeout_sec: Time in seconds to wait until file is getting saved
+    """
+    ipython_display = import_module("IPython.display")
+    log.debug("Saving notebook: %s", file_path)
+    deadline = time.time() + timeout_sec
+    start_md5 = hashlib.md5(open(file_path, "rb").read()).hexdigest()
+    ipython_display.display(
+        ipython_display.Javascript("IPython.notebook.save_checkpoint();")
+    )
+    current_md5 = start_md5
+    while (start_md5 == current_md5) and (time.time() < deadline):
+        log.debug("Waiting for the notebook to finish saving")
+        time.sleep(1)
+        current_md5 = hashlib.md5(open(file_path, "rb").read()).hexdigest()
+
+
 def get_jupyter_notebook_path():
     latest_session = get_latest_jupyter_session()
     server_info = get_jupyter_server_info()
@@ -142,7 +230,7 @@ def get_jupyter_notebook_path():
     )
 
 
-def get_colab_notebook_path():
+def get_colab_notebook_via_gdrive():
     mount_path = mount_google_drive()
     return os.path.join(
         mount_path,
@@ -252,3 +340,22 @@ def get_jupyter_server_endpoint() -> Tuple[str, str]:
         server_info.get("url", get_default_jupyter_api_session_host()),
         server_info.get("token", ""),
     )
+
+
+def show_warning_to_use_magic_command(reason: str = "") -> None:
+    """Show warning message to use `%aicrowd` magic command with complete code
+
+    Some features won't work with !aicrowd. `%aicrowd` magic command can be used to save
+    notebook inside jupyter notebook/jupyterLab environment and also to get notebook directly from frontend without
+    mounting the drive in colab environment.
+
+    Args:
+        reason: A string which describes the reason to use `%aicrowd` command over `!aicrowd`
+    """
+    description = (
+        f"`%aicrowd` magic command can be used to save the notebook inside jupyter notebook/jupyterLab environment "
+        f"and also to get the notebook directly from the frontend without mounting the drive in colab environment. "
+        f"You can use magic command {reason} and submit using the code below:\n "
+    )
+    code = "\n".join(["%load_ext aicrowd.magic", "%aicrowd " + " ".join(sys.argv[1:])])
+    warnings.warn(description + code)
