@@ -1,13 +1,14 @@
 import json
 import os
 import shutil
+import tempfile
 
 from aicrowd.notebook.clean import clean_notebook
 from aicrowd.notebook.helpers import (
     get_jupyter_notebook_path,
     get_kernel_from_language,
-    get_notebook_path,
     write_aicrowd_config,
+    get_notebook,
 )
 from aicrowd.notebook.split import split_notebook
 from aicrowd.notebook import exceptions
@@ -49,7 +50,17 @@ def bundle_submission(
     if len(os.listdir(assets_dir)) == 0:
         print("WARNING: Assets directory is empty")
 
-    print(f"Using notebook: {get_notebook_path()} for submission...")
+    notebook, notebook_name = get_notebook(notebook_path=notebook_name)
+    if isinstance(notebook, dict):
+        # Create a temporary file to save the notebook JSON
+        temp_notebook_file = tempfile.NamedTemporaryFile(mode="w+")
+        json.dump(notebook, temp_notebook_file)
+        temp_notebook_file.flush()
+        notebook_path = temp_notebook_file.name
+    else:
+        notebook_path = notebook
+
+    print(f"Using notebook: {notebook_name} for submission...")
 
     # Remove existing files from submission directoru
     if os.path.exists(SUBMISSION_DIR):
@@ -59,9 +70,7 @@ def bundle_submission(
 
     # Clean sensitive stuff from notebook
     cleaned_notebook_path = os.path.join(SUBMISSION_DIR, "original_notebook.ipynb")
-    if notebook_name is None:
-        notebook_name = get_notebook_path()
-    clean_notebook(notebook_name, cleaned_notebook_path)
+    clean_notebook(notebook_path, cleaned_notebook_path)
 
     # Breaks the notebook into {install, train, predict}.ipynb
     split_notebook(cleaned_notebook_path, SUBMISSION_DIR)
