@@ -1,15 +1,15 @@
 """
 Top level helpers
 """
-import io
 import os
 import subprocess
+import sys
 import uuid
 from abc import ABC
+from functools import wraps
 from typing import Any
 
 import click
-from rich.console import Console
 from rich.progress import (
     BarColumn,
     DownloadColumn,
@@ -21,6 +21,8 @@ from rich.progress import (
 )
 from tqdm.auto import tqdm
 
+from aicrowd.errors import UNKNOWN_ERROR
+from aicrowd.exceptions import CLIException
 from aicrowd.utils.jupyter import is_jupyter
 
 
@@ -202,3 +204,32 @@ class TqdmProgressBar(ProgressBar):
 
     def close(self, progress_bar_id: str):
         self.bars[progress_bar_id].close()
+
+
+def exception_handler(f):
+    """
+    try/except wrapper for CLI function calls
+    """
+
+    @wraps(f)
+    def _exception_handler(*args, **kwargs):
+        try:
+            f(*args, **kwargs)
+        except CLIException as e:
+            if not e.message:  # covers both "" and None
+                e.message = (
+                    "An unknown error occured. Please set verbosity to check full logs."
+                )
+
+            click.echo(str(e))
+            sys.exit(e.exit_code)
+        except Exception as e:
+            click.echo(click.style("An unexpected error occured!", fg="red"))
+            print(str(e))
+            click.echo(
+                "To get more information, you can run this command with -v.\n"
+                + "To increase level of verbosity, you can go upto -vvvvv"
+            )
+            sys.exit(UNKNOWN_ERROR)
+
+    return _exception_handler
